import 'dart:async';
import 'package:graphql_flutter/graphql_flutter.dart';
import 'package:http/http.dart' as Http;

typedef HttpResponseHandler = void Function(Response?, Http.Response);

class GraphQLInterceptHttpCodeLink extends Link {
  HttpResponseHandler httpResponseHandler;
  late StreamController<Response> controller;
  late Request gqlRequest;
  NextLink? forward;

  GraphQLInterceptHttpCodeLink({required this.httpResponseHandler});

  @override
  Stream<Response> request(Request request, [NextLink? forward]) {
    gqlRequest = request;
    this.forward = forward;
    controller = StreamController<Response>(onListen: onListen);

    return controller.stream;
  }

  void onListen() async {
    await controller.addStream(forward!(gqlRequest).handleError((error) {
      if(error != null && error is HttpLinkServerException){
        Http.Response httpResponse = error.response;
        if (httpResponse != null) {
            httpResponseHandler(null, httpResponse);
        }
      }
      throw error;
    }
    ));

    await controller.close();
  }
}


